Method: Autoproj::PackageManagers::ShellScriptManager#auto_install_cmd

Defined in:
lib/autoproj/package_managers/shell_script_manager.rb

#auto_install_cmdString (readonly)

Command line used by autoproj to install packages

Since it is to be used for automated install by autoproj, it should not require any interaction with the user. When generating the command line, the %s slot is replaced by the quoted package name(s).

Returns:

  • (String)

    a command line pattern that allows to install packages without user interaction. It is used when a package should be installed by autoproj automatically



80
81
82
# File 'lib/autoproj/package_managers/shell_script_manager.rb', line 80

def auto_install_cmd
  @auto_install_cmd
end