Method: Autoproj::PackageManagers::ShellScriptManager#initialize

Defined in:
lib/autoproj/package_managers/shell_script_manager.rb

#initialize(ws, needs_locking, user_install_cmd, auto_install_cmd, needs_root = true) ⇒ ShellScriptManager

Returns a new instance of ShellScriptManager.

Parameters:

  • names (Array<String>)

    the package managers names, see #names

  • needs_locking (Boolean)

    whether this package manager can be started by two separate autoproj instances at the same time. See #needs_locking?

  • user_install_cmd (String)

    the user-visible command line. See #user_install_cmd

  • auto_install_cmd (String)

    the command line used by autoproj itself, see #auto_install_cmd.

  • needs_root (Boolean) (defaults to: true)

    if the command lines should be started as root or not. See #needs_root?



103
104
105
106
107
108
109
110
# File 'lib/autoproj/package_managers/shell_script_manager.rb', line 103

def initialize(ws, needs_locking, user_install_cmd,
    auto_install_cmd, needs_root = true)
    super(ws)
    @needs_locking = needs_locking
    @user_install_cmd = user_install_cmd
    @auto_install_cmd = auto_install_cmd
    @needs_root = needs_root
end