Method: Autoproj::PackageManagers::ShellScriptManager#needs_locking?

Defined in:
lib/autoproj/package_managers/shell_script_manager.rb

#needs_locking?Boolean

Whether two autoproj instances can run this package manager at the same time

This declares if this package manager cannot be used concurrently. If it is the case, autoproj will ensure that there is no two autoproj instances running this package manager at the same time

Returns:

  • (Boolean)

See Also:



52
53
54
# File 'lib/autoproj/package_managers/shell_script_manager.rb', line 52

def needs_locking?
    @needs_locking
end