Method: Autoproj::PackageManagers::ShellScriptManager#user_install_cmd

Defined in:
lib/autoproj/package_managers/shell_script_manager.rb

#user_install_cmdString (readonly)

Command line displayed to the user to install packages

When generating the command line, the %s slot is replaced by the quoted package name(s).

Returns:

  • (String)

    a command line pattern that allows to install packages with user interaction. It is displayed to the user when it chose to not let autoproj install packages for this package manager automatically



90
91
92
# File 'lib/autoproj/package_managers/shell_script_manager.rb', line 90

def user_install_cmd
  @user_install_cmd
end