Method: Autoproj::PackageManagers::ShellScriptManager#user_install_cmd
- Defined in:
- lib/autoproj/package_managers/shell_script_manager.rb
#user_install_cmd ⇒ String (readonly)
Command line displayed to the user to install packages
When generating the command line, the %s slot is replaced by the quoted package name(s).
90 91 92 |
# File 'lib/autoproj/package_managers/shell_script_manager.rb', line 90 def user_install_cmd @user_install_cmd end |